module CURIC
  class << self
    attr_accessor :tools_command
  end
  @tools_command ||= UI.menu('Tools').add_submenu('Curic')

  module Gizmo
    UI.messagebox('Curic Gizmo requires SketchUp 2023 or newer.') if Sketchup.version.to_i < 23

    class << self
      attr_accessor(:settings, :observer)
    end

    PATH_R = File.join(File.dirname(__FILE__), 'Resources')

    GLOBAL_ORIENTATION = 0
    LOCAL_ORIENTATION = 1

    Sketchup.require("#{FILENAMESPACE}/utils")
    Sketchup.require("#{FILENAMESPACE}/observer")
    Sketchup.require("#{FILENAMESPACE}/overlay")
    
    # Load TT_Lib2 if not already loaded
    begin
      require 'TT_Lib2/core.rb'
      Sketchup.require("#{FILENAMESPACE}/gizmo")
    rescue LoadError => e
      UI.messagebox("Curic Gizmo requires TT_Lib².\nPlease download and install it from Extension Warehouse.\n\n#{e.message}")
    end

    def self.gizmo_orientation
      Sketchup.read_default(PLUGIN_NAME, 'orientation', GLOBAL_ORIENTATION)
    end

    def self.gizmo_orientation=(value)
      Sketchup.write_default(PLUGIN_NAME, 'orientation', value)
    end

    file = __FILE__
    unless file_loaded? file
      CURIC.tools_command.add_item('Toggle Gizmo') { toggle_gizmo }
      @observer = GizmoObserver.new
      
      file_loaded(file)
    end
  end
end
