module CURIC::Gizmo
  class GizmoObserver
    def initialize
      Sketchup.add_observer AppOb.new(self)
    end

    def selection_changed(selection)
      return unless PLUGIN.active_overlay && PLUGIN.active_overlay.enabled?

      PLUGIN.active_overlay.selection_changed(selection)
    end

    def tool_changed(tool_name)
      return unless PLUGIN.active_overlay && PLUGIN.active_overlay.enabled?

      PLUGIN.active_overlay.tool_changed(tool_name)
    end

    def on_transaction
      return unless PLUGIN.active_overlay && PLUGIN.active_overlay.enabled?

      PLUGIN.active_overlay.update_gizmo
    end

    class AppOb < Sketchup::AppObserver
      def initialize(observer = nil)
        @observer = observer

        begin
          attach_observers(Sketchup.active_model)
        rescue StandardError
        end
      end

      def onNewModel(model)
        attach_observers(model)
      end

      def onOpenModel(model)
        attach_observers(model)
      end

      def onActivateModel(model)
        overlay = model.overlays.to_a.find { |o| o.is_a?(CURIC::Gizmo::GizmoOverlay) }

        # Overlay
        return if overlay

        add_overlay(model)
      end

      private

      def attach_observers(model)
        model.add_observer(GizmoModelObserver.new(@observer))
        model.tools.add_observer(GizmoToolsOb.new(@observer))
        model.selection.add_observer(GizmoSelectionObserver.new(@observer))
        
        # Overlay
        return if model.overlays.to_a.find { |o| o.is_a?(CURIC::Gizmo::GizmoOverlay) }

        add_overlay(model)
      end

      def add_overlay(model)
        model.overlays.add(CURIC::Gizmo::GizmoOverlay.new)
      end
    end
    
    class GizmoModelObserver < Sketchup::ModelObserver
      def initialize(observer)
        @observer = observer
      end

      def onTransactionCommit(_model)
        on_transaction
      end

      def onTransactionRedo(_model)
        on_transaction
      end

      def onTransactionUndo(_model)
        on_transaction
      end

      def on_transaction
        @observer.on_transaction
      end
    end

    class GizmoSelectionObserver < Sketchup::SelectionObserver
      def initialize(observer)
        @observer = observer
      end

      def onSelectionBulkChange(selection)
        selection_changed(selection)
      end

      def onSelectionAdded(selection, _entity)
        selection_changed(selection)
      end

      def onSelectionCleared(selection)
        selection_changed(selection)
      end

      def selection_changed(selection)
        @observer.selection_changed(selection)
      end
    end

    class GizmoToolsOb < Sketchup::ToolsObserver
      def initialize(observer)
        @observer = observer
      end

      def onActiveToolChanged(_model, tool_name, _tool_id)
        tool_name = fix_mac_tool_name(tool_name)

        @observer.tool_changed(tool_name)
      end

      def fix_mac_tool_name(tool_name)
        if tool_name == 'eTool'
          tool_name = 'ScaleTool'
        elsif tool_name == 'ool'
          tool_name = 'MoveTool'
        elsif tool_name == 'onentCSTool'
          tool_name = 'ComponentCSTool'
        elsif tool_name == 'PullTool'
          tool_name = 'PushPullTool'
        end
        tool_name
      end
    end
  end
end
