module CURIC::Gizmo
  module ShapeGeom
    extend self

    def rectangle_points(origin = ORIGIN, width, height)
      rect = [
        Geom::Point3d.new(origin.x - width / 2.0, origin.y - height / 2.0, 0), 
        Geom::Point3d.new(origin.x + width / 2.0, origin.y - height / 2.0, 0), 
        Geom::Point3d.new(origin.x + width / 2.0, origin.y + height / 2.0, 0), 
        Geom::Point3d.new(origin.x - width / 2.0, origin.y + height / 2.0, 0) 
      ]

      rect
    end 
    
    def circle_line_width(center, radius, sides, size = 2)
      outer = circle_points(center, radius + (size / 2.0), sides)
      inner = circle_points(center, radius - (size / 2.0), sides)

      Geom.tesselate(outer, inner.reverse)
    end

    def line_width(line, size = 2, sides = 6)
      raise "Size must be > 1" if size < 2

      pts = []
      p_start, p_end = line
      vec_edge = p_start.vector_to(p_end)
      vec_cr = Z_AXIS.cross(vec_edge)
      vec_cr.length = size / 2.0
      
      # body
      pts << p_start.offset(vec_cr)
      
      angle = 360.0 / (sides * 2)
      trans = Geom::Transformation.rotation(p_start, Z_AXIS, angle.degrees)
      1.upto(sides) { pts << pts.last.transform(trans) }
      pts << pts.last.offset(vec_edge)
      
      trans = Geom::Transformation.rotation(p_end, Z_AXIS, angle.degrees)
      1.upto(sides) { pts << pts.last.transform(trans) }

      pts
    end

    def square_points(origin, size)
      rectangle_points(origin, size, size)
    end

    def square_uvs
      [
        Geom::Point3d.new(0, 0, 0),
        Geom::Point3d.new(1, 0, 0), 
        Geom::Point3d.new(1, 1, 0), 
        Geom::Point3d.new(0, 1, 0)
      ].reverse
    end

    def circle_points(center = ORIGIN, radius = 1, sides = 24)
      vecx = X_AXIS.clone
      vecx.length = radius
      circle = [center.offset(vecx)]
      tr = Geom::Transformation.rotation(center, Z_AXIS, (360.0 / sides).degrees)
      0.upto(sides - 1) { circle.push(circle.last.transform(tr)) }
    
      circle
    end

    def circle_uvs(sides = 24)
      circle = [ORIGIN.offset(X_AXIS)]
      tr = Geom::Transformation.rotation(ORIGIN, Z_AXIS, (360 / sides).degrees)
      0.upto(sides - 1) { circle.push(circle.last.transform(tr)) }
      uvs = circle.map { |pt| pt.offset(X_AXIS + Y_AXIS).to_a.map { |i| i / 2.0 } }

      uvs.reverse
    end

    def create_box(center, d)
      x, y, z = center.to_a
      half_d = d / 2.0

      vertices = [
        [x - half_d, y - half_d, z + half_d], # bottom-back-left
        [x - half_d, y - half_d, z - half_d], # bottom-front-left
        [x + half_d, y - half_d, z - half_d], # bottom-front-right
        [x + half_d, y - half_d, z + half_d], # bottom-back-right
        [x - half_d, y + half_d, z + half_d], # top-back-left
        [x - half_d, y + half_d, z - half_d], # top-front-left
        [x + half_d, y + half_d, z - half_d], # top-front-right
        [x + half_d, y + half_d, z + half_d]  # top-back-right
      ]

      faces = [
        [vertices[0], vertices[1], vertices[2], vertices[3]], # bottom
        [vertices[4], vertices[5], vertices[6], vertices[7]], # top
        [vertices[0], vertices[4], vertices[7], vertices[3]], # back
        [vertices[1], vertices[5], vertices[4], vertices[0]], # left
        [vertices[2], vertices[6], vertices[5], vertices[1]], # front
        [vertices[3], vertices[7], vertices[6], vertices[2]]  # right
      ]

      faces
    end
  end
end
